/*
   MemAid, a memorisation tool.
   Copyright (C) 2003 David Calinski
   Portions of the code: Copyright (c) 1996 Karsten Kutza

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307  USA
*/

#include <ctype.h>
#include "core/memaid.h"
#include "defs.h"

#define GRADE_THRESHOLD 4 /* used in final drill */
#define DEFAULT_DATA_DIR "data/" /* you may want to change it (e.g. put full absolute path) */
#define API_STATE_FILE	pathed("api_state.bin")


/*
'state'   command   description

  1        rep       repetition
  2        nfq       new elements from queue.txt
  3        nwX       new element
----- NOT USED----  4        fer       force ealier repetitiotions
  5        dri       final drill
  6  [not used]
  7        edi       edit element
  8        del       delete element
  9        ans       show answer
  10       grX       grade (X = 0-5, 5 is the best)
  11       today     reps still to do today 
  12       drill     reps still to be drilled today potentially 

        states 1-5 (and element data) are remembered in api_state.bin file,
	so if next command is "del", "ans" or "grX" api "knows" what it's related to


api_responses     comment
  OK               everything is ok, question/answer begins in next line
  NO               no element found
  ER               an error. description of error is in next line
*/


static void print_usage (void) {
	printf ("API can work in two modes.\n");
	printf ("*** FIRST MODE: ***\n");
	printf ("Usage:\n");
	printf ("api rep|nfq|nwX|dri|edi|del|ans|gr0|gr1|gr1|gr2|gr3|gr4|gr5|today|drill\n");
	
	printf ("\"rep\" - \"repetitions\" mode\n");
	printf ("\"nfq\" - \"getting new elements from queue.txt\" mode\n");
	printf ("\"nwX\" - \"entering new item\" mode, X is grade\n\t"
		"(MemAid reads from stdin (up to new line) new q, then new a)\n");
	printf ("\"dri\" - \"final drill\" mode\n");
	printf ("\"edi\" - \"edit\" mode (MemAid reads from stdin (up to new line) new q, then new a)\n");
	printf ("\"del\" - \"delete current item from the learning process\"\n");
	printf ("\"ans\" - \"show answer\"\n");
	printf ("\"grX\" - a grade to last answer, (X = 0-5; 5 is the best)\n");
	printf ("\"drill\" - show how many items are in final drill\n");
	printf ("\"today\" - how many items still left to repeat today\n");
	
	printf ("Responses:\n");
	printf ("\"OK\" and answer/question starts in next line (or new interval is printed there)\n");
	printf ("\"NO\" - no element found (for repetition, final drill, etc.)\n");
	printf ("\"ER\" - error. error description in next line\n");
	printf ("\n");

	printf ("*** SECOND MODE (only input/output to/from ANN): ***\n");
	printf ("api 1 2 3 4\n");
	printf ("Where:\n");
	printf ("\"1\" - \"interval_anticipated_by_ann\" in days (0-%d)\n", MA_MAX_N_OF_RPTS);
	printf ("\"2\" - \"real_interval\" in days (0-%d)\n", MA_MAX_N_OF_RPTS);
	printf ("\"3\" - \"number of repetitions so far\" (0-%d)\n", MA_MAX_INTERVAL);
	printf ("\"4\" - \"grade\" (0-5, where 5 is the best)\n");
	printf ("output will be new optimal interval computed by ANN\n");
	return;
}


static char q[MAX_Q_LEN], a[MAX_A_LEN];
u_short state;


static u_short on_grade (u_short grade) {
	struct elem *el;
	
	if (state == 0) {
		puts ("ER\ngrade for what?");
		return 0;
	}
	
	el = ma_find_el (q, a);
	if (el == NULL) {
		puts ("ER\nelement doesn't exist anymore, or I can't access it.");
		return 0;
	}

	switch (state) {
		case 1:
		case 5:
			return ma_move_el (el, grade);
		case 2:
			ma_get_new_el (MA_DELETE_IT, q, a);
			return ma_make_new_el (grade, q, a);
		default:
			puts ("ER\nunknown error - default case in on_grade()\n");
			return 0;
	}
}

static int on_delete (void) {
	struct elem *el;

	el = ma_find_el (q, a);
	if (el == NULL) {
		puts ("ER\nelement doesn't exist anymore, or I can't access it.");
		return 1;
	}
	ma_delete_el(el);
	return 0;
}

static int on_edit (void) {
	struct elem *el;
	el = ma_find_el (q, a);
	if (el == NULL) {
		puts ("ER\nelement doesn't exist anymore, or I can't access it.");
		return 1;
	}

	fgets (q, MAX_Q_LEN, stdin);
	fgets (a, MAX_A_LEN, stdin);
	el->q = (char*) realloc (el->q, sizeof(char) * (strlen(q)+1));
	el->a = (char*) realloc (el->a, sizeof(char) * (strlen(a)+1));
	strcpy (el->q, q);
	strcpy (el->a, a);
	
	return 0;
}

static int on_new (u_short grade) {
	fgets (q, MAX_Q_LEN, stdin);
	fgets (a, MAX_A_LEN, stdin);
	return ma_make_new_el (grade, q, a);
}

int main (int argc, char *argv[]) {
	u_short grade, i = 0, j = 0;
	struct elem *el = NULL;
	char *path_to_data_dir = NULL;
	FILE *fstate;
	
	memset (q, '\0', MAX_Q_LEN);
	memset (a, '\0', MAX_A_LEN);
	state = 0;

	while (++i < argc) {
		if (strcmp (argv[i], "rep") == 0) {
			j = 1;
			break;
		}
		if (strcmp (argv[i], "nfq") == 0) {
			j = 2;
			break;
		}
		if (strncmp (argv[i], "nw", 2) == 0) {
			j = 3;
			grade = argv[i][2] - '0';
			if (grade > 5) {
				printf ("grade = %d?\n", grade);
				exit (1);
			}
			break;
		}
		if (strcmp (argv[i], "dri") == 0) {
			j = 5;
			break;
		}
		if (strcmp (argv[i], "edi") == 0) {
			j = 7;
			break;
		}
		if (strcmp (argv[i], "del") == 0) {
			j = 8;
			break;
		}
		if (strcmp (argv[i], "ans") == 0) {
			j = 9; 
			break;
		}
		if (strncmp (argv[i], "gr", 2) == 0) {
			j = 10;
			grade = argv[i][2] - '0';
			if (grade > 5) {
				printf ("grade = %d?\n", grade);
				exit (1);
			}
			break;
		}
	        if (strcmp (argv[i], "today") == 0) {
			j = 11;
			break;
		}
	        if (strcmp (argv[i], "drill") == 0) {
			j = 12;
			break;
		}
		if (argv[i][0] == 'd') {
			path_to_data_dir = &argv[i][1];
			continue;
		}
	}
	if (j == 0 && argc != 5) {
		print_usage();
		exit (1);
	}

        if (path_to_data_dir != NULL)
		i = (u_short) ma_init (path_to_data_dir);
	else
		i = (u_short) ma_init (DEFAULT_DATA_DIR);
	
	if (i & DIR_NOT_ACCESSIBLE)
		exit (1); /* error msg already printed by ma_init() */
	if (i & MEMAID_ALREADY_RUNNING)
		exit (1); /* error msg already printed */

	if (j != 9) {
	   fstate = fopen (API_STATE_FILE, "rb");
	   if (fstate) {
		if (fread (&state, sizeof(u_short), 1, fstate) != 1) {
			puts ("ER\nAn error while reading data from api_state.bin file");
			exit (1);
		}
		if (fread (&i, sizeof(u_short), 1, fstate) == 1) {
			fread (q, sizeof(char), i, fstate);
			fread (&i, sizeof(u_short), 1, fstate);
			fread (a, sizeof(char), i, fstate);
		}
		else {
			puts ("ER\nAn error while reading data from api_state.bin file");
			exit (1);
		}
		fclose (fstate);
		remove (API_STATE_FILE);
	   }
	}
   

	switch (j) {
		case 0:
			printf ("%d\n", ma_new_interval ((u_short)atoi (argv[1]),
					 (u_short)atoi (argv[2]),
					 (u_short)atoi (argv[3]),
					 (u_short)atoi (argv[4])));
			state = 0;
			break;
		case 1:
			if ((el = ma_el_to_repeat ()) != NULL) {
				printf ("OK\n%s\n", el->q);
				state = 1;
			}
			else {
				state = 0;
				puts ("NO");
			}
			break;
		case 2:
			if (!ma_get_new_el (MA_JUST_RETURN_IT, q, a)) {
				printf ("OK\n%s\n", el->q);
				state = 2;
			}
			else {
				state = 0;
				puts ("NO");
			}
			break;
		case 3:
			printf ("OK\n%d\n", on_new (grade));
			break;
		case 5:
			if ((el = ma_final_drill (GRADE_THRESHOLD))) {
				printf ("OK\n%s\n", el->q);
				state = 5;
			}
			else {
				state = 0;
				puts ("NO");
			}
			break;
		case 7:
			if (state == 0) {
				puts ("ER\nEdit what element?");
				break;
			}
			if (on_edit() == 0)
				puts ("OK");
			break;
		case 8:
			if (state == 0) {
				puts ("ER\nDelete what element?");
				break;
			}
			if (on_delete() == 0)
				puts ("OK");
			break;
		case 9:
			if (state == 0) {
				puts ("ER\nshow what answer?");
				break;
			}
			state = 9;
			printf ("OK\n%s\n", a);
			break;
		case 10:
			i = on_grade(grade);
			if (i > 0)
				printf ("OK\n%hu\n", i);
			state = 0;
			break;
 	        case 11:
	                i=ma_n_of_rpts_for_today();
	                printf ("OK\nToday: still %d reps\n",i);
	                break;
 	        case 12:
	                i=ma_rpts_drill(GRADE_THRESHOLD);
	                printf ("OK\nToday:  %d to drill\n",i);
	                break;
	}
        
	if (state && (state != 9)) {
		fstate = fopen (API_STATE_FILE, "wb");
		fwrite (&state, sizeof(u_short), 1, fstate);
		i = strlen (el->q) + 1;
		fwrite (&i, sizeof(u_short), 1, fstate);
		fwrite (el->q, sizeof(char), i, fstate);
		i = strlen (el->a) + 1;
		fwrite (&i, sizeof(u_short), 1, fstate);
		fwrite (el->a, sizeof(char), i, fstate);
		fclose (fstate);
	}
	
	ma_deinit();
	return 0;
}

